@@ -28,6 +28,9 @@
 #ifdef HAVE_PWD_H
 #include <pwd.h>
 #endif
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
 
 #include <stdlib.h>
 
@@ -241,6 +244,11 @@ main (int    argc,
   /* set the program name */
   g_set_prgname (G_LOG_DOMAIN);
 
+#ifdef G_OS_UNIX
+  if (nice (19) != 19)
+    g_warning (_("Couldn't change nice value of process."));
+#endif
+
 #ifdef DEBUG
   /* if something doesn't work, fix your code instead! */
   g_log_set_always_fatal (G_LOG_LEVEL_CRITICAL);
@@ -249,8 +257,10 @@ main (int    argc,
   /* set the application name. Translators: Don't translate "Tumbler". */
   g_set_application_name (_("Tumbler Thumbnailing Service"));
 
+#if !GLIB_CHECK_VERSION (2, 36, 0)
   /* initialize the GLib type system */
   g_type_init ();
+#endif
 
 #if !GLIB_CHECK_VERSION (2, 32, 0)
   /* initialize threading system */
